// Copyright (c) 2019-2020 The Authors.
// This file is part of ngx-highlight.
//
// SPDX-License-Identifier: BSD-3-Clause
import { __decorate, __metadata } from "tslib";
import * as _ from 'lodash';
import { Directive, ElementRef, HostListener, Input, Renderer2, OnInit, OnChanges, SimpleChanges, OnDestroy, } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';
var HighlightDirective = /** @class */ (function () {
    function HighlightDirective(_renderer, _el) {
        this._renderer = _renderer;
        this._el = _el;
        // Automatic unsubscription when the component is destroyed
        this._ngUnsubscribe$ = new Subject();
        this._style = {
            hover: 'none',
            focus: 'none',
            debounceTime: 50,
        };
        this._hovered$ = new BehaviorSubject(false);
        this._focused$ = new BehaviorSubject(false);
    }
    HighlightDirective.prototype.ngOnInit = function () {
        var _this = this;
        this._updateStyleConfig();
        this._hovered$
            .pipe(debounceTime(50), takeUntil(this._ngUnsubscribe$))
            .subscribe(function (status) {
            if (_this._isHovered != status) {
                _this._isHovered = status;
                // Do not apply the hover effect if the element is focused
                if (_this._isHovered && !_this._isFocused)
                    _this._applyEffect('hover');
                else
                    _this._removeEffect('hover');
            }
        });
        this._focused$
            .pipe(debounceTime(50), takeUntil(this._ngUnsubscribe$))
            .subscribe(function (status) {
            if (_this._isFocused != status) {
                _this._isFocused = status;
                if (_this._isFocused) {
                    _this._applyEffect('focus');
                    // If the element is hovered, then remove the hover effect
                    if (_this._isHovered)
                        _this._removeEffect('hover');
                }
                else {
                    _this._removeEffect('focus');
                    // If the element is hovered, then apply the hover effect
                    if (_this._isHovered)
                        _this._applyEffect('hover');
                }
            }
        });
    };
    HighlightDirective.prototype.ngOnChanges = function (changes) {
        if (changes['highlightStyle']) {
            this._updateStyleConfig();
        }
    };
    HighlightDirective.prototype.ngOnDestroy = function () {
        // The emitted value is not important and not to be used, but is here
        // simply because it does not make sense to call next() with no value...
        this._ngUnsubscribe$.next(true);
        // Security measure to avoid memory leaks, as calling complete on
        // the source stream will remove the references to all the subscribed
        // observers, allowing the garbage collector to eventually dispose any
        // non unsubscribed Subscription instance.
        this._ngUnsubscribe$.complete();
    };
    HighlightDirective.prototype._updateStyleConfig = function () {
        if (this.highlightStyle)
            _.assign(this._style, this.highlightStyle);
        if (this._style.debounceTime < 0)
            this._style.debounceTime = 0;
    };
    HighlightDirective.prototype.onMouseEnter = function () {
        this._hovered$.next(true);
    };
    HighlightDirective.prototype.onMouseLeave = function () {
        this._hovered$.next(false);
    };
    HighlightDirective.prototype.onFocusIn = function () {
        this._focused$.next(true);
    };
    HighlightDirective.prototype.onFocusOut = function () {
        this._focused$.next(false);
    };
    HighlightDirective.prototype._applyEffect = function (effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-shadow");
                break;
            case 'outline':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-outline");
                break;
            case 'background':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-background");
                break;
            // Default is 'none'
            default:
        }
    };
    HighlightDirective.prototype._removeEffect = function (effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-shadow");
                break;
            case 'outline':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-outline");
                break;
            case 'background':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-background");
                break;
            // Default is 'none'
            default:
        }
    };
    HighlightDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input('hcHighlight'),
        __metadata("design:type", Object)
    ], HighlightDirective.prototype, "highlightStyle", void 0);
    __decorate([
        HostListener('mouseenter'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseEnter", null);
    __decorate([
        HostListener('mouseleave'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseLeave", null);
    __decorate([
        HostListener('focusin'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusIn", null);
    __decorate([
        HostListener('focusout'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusOut", null);
    HighlightDirective = __decorate([
        Directive({
            selector: '[hcHighlight]',
        }),
        __metadata("design:paramtypes", [Renderer2, ElementRef])
    ], HighlightDirective);
    return HighlightDirective;
}());
export { HighlightDirective };
//# sourceMappingURL=highlight.directive.js.map